import { authOptions } from "@/app/auth";
import {
	Table,
	TableBody,
	TableCaption,
	TableCell,
	TableFooter,
	TableHead,
	TableHeader,
	TableRow,
} from "@/components/ui/table";
import { getDevices } from "@/queries/devices";
import { tryCatch } from "@/utils/tryCatch";
import { getServerSession } from "next-auth";
import ClickableRow from "./clickable-row";
import DeviceCard from "./device-card";
import Pagination from "./pagination";

export async function DevicesTable({
	searchParams,
	parentalControl,
}: {
	searchParams: Promise<{
		query: string;
		page: number;
		sortBy: string;
	}>;
	parentalControl?: boolean;
}) {
	const session = await getServerSession(authOptions);
	const isAdmin = session?.user?.is_superuser;
	const query = (await searchParams)?.query || "";

	const [error, devices] = await tryCatch(getDevices({ query: query }));
	if (error) {
		return <pre>{JSON.stringify(error, null, 2)}</pre>;
	}
	const { meta, links, data } = devices;
	return (
		<div>
			{data.length === 0 ? (
				<div className="h-[calc(100svh-400px)] flex flex-col items-center justify-center my-4">
					<h3>No devices yet.</h3>
				</div>
			) : (
				<>
					<div className="hidden sm:block">
						<Table className="overflow-scroll">
							<TableCaption>Table of all devices.</TableCaption>
							<TableHeader>
								<TableRow>
									<TableHead>Device Name</TableHead>
									<TableHead>MAC Address</TableHead>
									<TableHead>#</TableHead>
								</TableRow>
							</TableHeader>
							<TableBody className="overflow-scroll">
								{data.map((device) => (
									<ClickableRow
										admin={isAdmin}
										key={device.id}
										device={device}
										parentalControl={parentalControl}
									/>
								))}
							</TableBody>
							<TableFooter>
								<TableRow>
									<TableCell colSpan={2}>
										{query.length > 0 && (
											<p className="text-sm text-muted-foreground">
												Showing {meta.total} locations for &quot;{query}
												&quot;
											</p>
										)}
									</TableCell>
									<TableCell className="text-muted-foreground">
										{meta.total} devices
									</TableCell>
								</TableRow>
							</TableFooter>
						</Table>
						<Pagination
							totalPages={meta.total / meta.per_page}
							currentPage={meta.current_page}
						/>
						<pre>{JSON.stringify(meta, null, 2)}</pre>
					</div>
					<div className="sm:hidden my-4">
						{data.map((device) => (
							<DeviceCard
								parentalControl={parentalControl}
								key={device.id}
								device={device}
							/>
						))}
					</div>
				</>
			)}
		</div>
	);
}
